package edu.dartmouth.cs.databasedemo;

import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import androidx.loader.app.LoaderManager;
import androidx.loader.content.Loader;

import android.util.Log;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.ListView;

import java.util.List;
import java.util.Random;

import edu.dartmouth.cs.databasedemo.model.Comment;

public class TestDatabaseActivity extends AppCompatActivity implements LoaderManager.LoaderCallbacks<List<Comment>> {

    private static final int ALL_COMMENTS_LOADER_ID =  1;
    private CommentsDataSource datasource;
    private ArrayAdapter<Comment> mAdapter;

    private String[] comments = new String[]{"Arsenal", "Arsenal", "Spurs", "are", "really suck!",
            "really suck!", "the best", "Arsenal", "the worst", "positively suck"};

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Log.d(CommentsListLoader.TAG, "onCreate(): Thread ID " + Thread.currentThread().getId());
        setContentView(R.layout.main);

        ActionBar actionBar = getSupportActionBar();

        if (actionBar != null) {
            actionBar.setDisplayShowHomeEnabled(true);
            actionBar.setSubtitle(" The Arsenal DB");
            actionBar.setIcon(R.drawable.ic_database);
        }

        ListView mListView = findViewById(R.id.list);
        datasource = new CommentsDataSource(this);
        datasource.open();

        // offload the query()
        // List<Comment> values = datasource.getAllComments();

        // Use the SimpleCursorAdapter to show the elements in a ListView
        mAdapter = new ArrayAdapter<>(this,
                android.R.layout.simple_list_item_1);

        mListView.setAdapter(mAdapter);

        LoaderManager mLoader = LoaderManager.getInstance(this);
        mLoader.initLoader(ALL_COMMENTS_LOADER_ID, null, this).forceLoad();

    }

    // Will be called via the onClick attribute of the buttons in main.xml
    public void onClick(View view) {
        Comment comment;
        switch (view.getId()) {

            case R.id.add:
                int nextInt = new Random().nextInt(comments.length);
                // Save the new comment to the database
                comment = datasource.createComment(comments[nextInt]);
                mAdapter.add(comment);
                break;

            case R.id.delete:
                if (mAdapter.getCount() > 0) {
                    comment = mAdapter.getItem(0);
                    datasource.deleteComment(comment);
                    mAdapter.remove(comment);
                }
                break;

            case R.id.deleteall:
                if (mAdapter.getCount() > 0) {
                    datasource.deleteAllComments();
                    mAdapter.clear();
                }
                break;
        }
        mAdapter.notifyDataSetChanged();
    }

    @Override
    protected void onResume() {
        datasource.open();
        super.onResume();
    }

    @Override
    protected void onPause() {
        datasource.close();
        super.onPause();
    }

    // Loader callbacks

    @NonNull
    @Override
    public Loader<List<Comment>> onCreateLoader(int id, @Nullable Bundle args) {
        Log.d(CommentsListLoader.TAG, "onCreateLoader(): Thread ID " + Thread.currentThread().getId());
        if (id == ALL_COMMENTS_LOADER_ID)
            return new CommentsListLoader(this);
        return null;
    }

    @Override
    public void onLoadFinished(@NonNull Loader<List<Comment>> loader, List<Comment> data) {
        Log.d(CommentsListLoader.TAG, "onLoadFinished(): Thread ID " + Thread.currentThread().getId());
        if (loader.getId() == ALL_COMMENTS_LOADER_ID) {
            if (data.size() > 0) {
                mAdapter.addAll(data);
            }
        }
    }

    @Override
    public void onLoaderReset(@NonNull Loader<List<Comment>> loader) {
        Log.d(CommentsListLoader.TAG, "onLoaderReset(): Thread ID " + Thread.currentThread().getId());
    }
}